@@ -50,17 +50,7 @@ where
     }
 
     #[cfg(feature = "iterator")]
-    pub fn prefix(&self, p: K::Prefix) -> Prefix<Vec<u8>, T> {
-        Prefix::new(self.namespace, &p.prefix())
-    }
-
-    #[cfg(feature = "iterator")]
-    pub fn sub_prefix(&self, p: K::SubPrefix) -> Prefix<Vec<u8>, T> {
-        Prefix::new(self.namespace, &p.prefix())
-    }
-
-    #[cfg(feature = "iterator")]
-    pub(crate) fn no_prefix(&self) -> Prefix<Vec<u8>, T> {
+    pub(crate) fn no_prefix_raw(&self) -> Prefix<Vec<u8>, T> {
         Prefix::new(self.namespace, &[])
     }
 
@@ -125,16 +115,16 @@ where
     T: Serialize + DeserializeOwned,
     K: PrimaryKey<'a>,
 {
-    pub fn sub_prefix_de(&self, p: K::SubPrefix) -> Prefix<K::SuperSuffix, T> {
+    pub fn sub_prefix(&self, p: K::SubPrefix) -> Prefix<K::SuperSuffix, T> {
         Prefix::new(self.namespace, &p.prefix())
     }
 
-    pub fn prefix_de(&self, p: K::Prefix) -> Prefix<K::Suffix, T> {
+    pub fn prefix(&self, p: K::Prefix) -> Prefix<K::Suffix, T> {
         Prefix::new(self.namespace, &p.prefix())
     }
 }
 
-// short-cut for simple keys, rather than .prefix(()).range(...)
+// short-cut for simple keys, rather than .prefix(()).range_raw(...)
 #[cfg(feature = "iterator")]
 impl<'a, K, T> Map<'a, K, T>
 where
@@ -143,11 +133,12 @@ where
     // Other cases need to call prefix() first
     K: PrimaryKey<'a>,
 {
-    /// while range assumes you set the prefix to one element and call range over the last one,
-    /// prefix_range accepts bounds for the lowest and highest elements of the Prefix we wish to
-    /// accept, and iterates over those. There are some issues that distinguish these to and blindly
-    /// casting to Vec<u8> doesn't solve them.
-    pub fn prefix_range<'c>(
+    /// While `range_raw` over a `prefix` fixes the prefix to one element and iterates over the
+    /// remaining, `prefix_range_raw` accepts bounds for the lowest and highest elements of the `Prefix`
+    /// itself, and iterates over those (inclusively or exclusively, depending on `PrefixBound`).
+    /// There are some issues that distinguish these two, and blindly casting to `Vec<u8>` doesn't
+    /// solve them.
+    pub fn prefix_range_raw<'c>(
         &self,
         store: &'c dyn Storage,
         min: Option<PrefixBound<'a, K::Prefix>>,
@@ -163,7 +154,7 @@ where
         Box::new(mapped)
     }
 
-    pub fn range<'c>(
+    pub fn range_raw<'c>(
         &self,
         store: &'c dyn Storage,
         min: Option<Bound>,
@@ -173,10 +164,10 @@ where
     where
         T: 'c,
     {
-        self.no_prefix().range(store, min, max, order)
+        self.no_prefix_raw().range_raw(store, min, max, order)
     }
 
-    pub fn keys<'c>(
+    pub fn keys_raw<'c>(
         &self,
         store: &'c dyn Storage,
         min: Option<Bound>,
@@ -186,7 +177,7 @@ where
     where
         T: 'c,
     {
-        self.no_prefix().keys(store, min, max, order)
+        self.no_prefix_raw().keys_raw(store, min, max, order)
     }
 }
 
@@ -196,11 +187,13 @@ where
     T: Serialize + DeserializeOwned,
     K: PrimaryKey<'a> + KeyDeserialize,
 {
-    /// while range_de assumes you set the prefix to one element and call range over the last one,
-    /// prefix_range_de accepts bounds for the lowest and highest elements of the Prefix we wish to
-    /// accept, and iterates over those. There are some issues that distinguish these to and blindly
-    /// casting to Vec<u8> doesn't solve them.
-    pub fn prefix_range_de<'c>(
+    /// While `range` over a `prefix` fixes the prefix to one element and iterates over the
+    /// remaining, `prefix_range` accepts bounds for the lowest and highest elements of the
+    /// `Prefix` itself, and iterates over those (inclusively or exclusively, depending on
+    /// `PrefixBound`).
+    /// There are some issues that distinguish these two, and blindly casting to `Vec<u8>` doesn't
+    /// solve them.
+    pub fn prefix_range<'c>(
         &self,
         store: &'c dyn Storage,
         min: Option<PrefixBound<'a, K::Prefix>>,
@@ -218,7 +211,7 @@ where
         Box::new(mapped)
     }
 
-    pub fn range_de<'c>(
+    pub fn range<'c>(
         &self,
         store: &'c dyn Storage,
         min: Option<Bound>,
@@ -229,10 +222,10 @@ where
         T: 'c,
         K::Output: 'static,
     {
-        self.no_prefix_de().range_de(store, min, max, order)
+        self.no_prefix().range(store, min, max, order)
     }
 
-    pub fn keys_de<'c>(
+    pub fn keys<'c>(
         &self,
         store: &'c dyn Storage,
         min: Option<Bound>,
@@ -243,10 +236,10 @@ where
         T: 'c,
         K::Output: 'static,
     {
-        self.no_prefix_de().keys_de(store, min, max, order)
+        self.no_prefix().keys(store, min, max, order)
     }
 
-    fn no_prefix_de(&self) -> Prefix<K, T> {
+    fn no_prefix(&self) -> Prefix<K, T> {
         Prefix::new(self.namespace, &[])
     }
 }
@@ -257,13 +250,12 @@ mod test {
     use serde::{Deserialize, Serialize};
     use std::ops::Deref;
 
-    #[cfg(feature = "iterator")]
-    use crate::U32Key;
-    use crate::U8Key;
     use cosmwasm_std::testing::MockStorage;
     #[cfg(feature = "iterator")]
     use cosmwasm_std::{Order, StdResult};
 
+    use crate::int_key::CwIntKey;
+
     #[derive(Serialize, Deserialize, PartialEq, Debug, Clone)]
     struct Data {
         pub name: String,
@@ -272,11 +264,13 @@ mod test {
 
     const PEOPLE: Map<&[u8], Data> = Map::new("people");
     #[cfg(feature = "iterator")]
-    const PEOPLE_ID: Map<U32Key, Data> = Map::new("people_id");
+    const PEOPLE_ID: Map<u32, Data> = Map::new("people_id");
+    #[cfg(feature = "iterator")]
+    const SIGNED_ID: Map<i32, Data> = Map::new("signed_id");
 
     const ALLOWANCE: Map<(&[u8], &[u8]), u64> = Map::new("allow");
 
-    const TRIPLE: Map<(&[u8], U8Key, &str), u64> = Map::new("triple");
+    const TRIPLE: Map<(&[u8], u8, &str), u64> = Map::new("triple");
 
     #[test]
     fn create_path() {
@@ -298,7 +292,7 @@ mod test {
         assert_eq!(b"john".to_vec().as_slice(), &key[9..13]);
         assert_eq!(b"maria".to_vec().as_slice(), &key[13..]);
 
-        let path = TRIPLE.key((b"john", 8u8.into(), "pedro"));
+        let path = TRIPLE.key((b"john", 8u8, "pedro"));
         let key = path.deref();
         // this should be prefixed(allow) || prefixed(john) || maria
         assert_eq!(
@@ -307,7 +301,7 @@ mod test {
         );
         assert_eq!(b"triple".to_vec().as_slice(), &key[2..8]);
         assert_eq!(b"john".to_vec().as_slice(), &key[10..14]);
-        assert_eq!(8u8.to_be_bytes(), &key[16..17]);
+        assert_eq!(8u8.to_cw_bytes(), &key[16..17]);
         assert_eq!(b"pedro".to_vec().as_slice(), &key[17..]);
     }
 
@@ -387,27 +381,87 @@ mod test {
         let mut store = MockStorage::new();
 
         // save and load on a triple composite key
-        let triple = TRIPLE.key((b"owner", 10u8.into(), "recipient"));
+        let triple = TRIPLE.key((b"owner", 10u8, "recipient"));
         assert_eq!(None, triple.may_load(&store).unwrap());
         triple.save(&mut store, &1234).unwrap();
         assert_eq!(1234, triple.load(&store).unwrap());
 
         // not under other key
         let different = TRIPLE
-            .may_load(&store, (b"owners", 10u8.into(), "ecipient"))
+            .may_load(&store, (b"owners", 10u8, "ecipient"))
             .unwrap();
         assert_eq!(None, different);
 
         // matches under a proper copy
-        let same = TRIPLE
-            .load(&store, (b"owner", 10u8.into(), "recipient"))
-            .unwrap();
+        let same = TRIPLE.load(&store, (b"owner", 10u8, "recipient")).unwrap();
         assert_eq!(1234, same);
     }
 
     #[test]
     #[cfg(feature = "iterator")]
-    fn range_simple_key() {
+    fn range_raw_simple_key() {
+        let mut store = MockStorage::new();
+
+        // save and load on two keys
+        let data = Data {
+            name: "John".to_string(),
+            age: 32,
+        };
+        PEOPLE.save(&mut store, b"john", &data).unwrap();
+
+        let data2 = Data {
+            name: "Jim".to_string(),
+            age: 44,
+        };
+        PEOPLE.save(&mut store, b"jim", &data2).unwrap();
+
+        // let's try to iterate!
+        let all: StdResult<Vec<_>> = PEOPLE
+            .range_raw(&store, None, None, Order::Ascending)
+            .collect();
+        let all = all.unwrap();
+        assert_eq!(2, all.len());
+        assert_eq!(
+            all,
+            vec![
+                (b"jim".to_vec(), data2.clone()),
+                (b"john".to_vec(), data.clone())
+            ]
+        );
+
+        // let's try to iterate over a range
+        let all: StdResult<Vec<_>> = PEOPLE
+            .range_raw(
+                &store,
+                Some(Bound::Inclusive(b"j".to_vec())),
+                None,
+                Order::Ascending,
+            )
+            .collect();
+        let all = all.unwrap();
+        assert_eq!(2, all.len());
+        assert_eq!(
+            all,
+            vec![(b"jim".to_vec(), data2), (b"john".to_vec(), data.clone())]
+        );
+
+        // let's try to iterate over a more restrictive range
+        let all: StdResult<Vec<_>> = PEOPLE
+            .range_raw(
+                &store,
+                Some(Bound::Inclusive(b"jo".to_vec())),
+                None,
+                Order::Ascending,
+            )
+            .collect();
+        let all = all.unwrap();
+        assert_eq!(1, all.len());
+        assert_eq!(all, vec![(b"john".to_vec(), data)]);
+    }
+
+    #[test]
+    #[cfg(feature = "iterator")]
+    fn range_simple_string_key() {
         let mut store = MockStorage::new();
 
         // save and load on two keys
@@ -467,7 +521,7 @@ mod test {
 
     #[test]
     #[cfg(feature = "iterator")]
-    fn range_de_simple_string_key() {
+    fn range_simple_integer_key() {
         let mut store = MockStorage::new();
 
         // save and load on two keys
@@ -475,81 +529,17 @@ mod test {
             name: "John".to_string(),
             age: 32,
         };
-        PEOPLE.save(&mut store, b"john", &data).unwrap();
+        PEOPLE_ID.save(&mut store, 1234, &data).unwrap();
 
         let data2 = Data {
             name: "Jim".to_string(),
             age: 44,
         };
-        PEOPLE.save(&mut store, b"jim", &data2).unwrap();
-
-        // let's try to iterate!
-        let all: StdResult<Vec<_>> = PEOPLE
-            .range_de(&store, None, None, Order::Ascending)
-            .collect();
-        let all = all.unwrap();
-        assert_eq!(2, all.len());
-        assert_eq!(
-            all,
-            vec![
-                (b"jim".to_vec(), data2.clone()),
-                (b"john".to_vec(), data.clone())
-            ]
-        );
-
-        // let's try to iterate over a range
-        let all: StdResult<Vec<_>> = PEOPLE
-            .range_de(
-                &store,
-                Some(Bound::Inclusive(b"j".to_vec())),
-                None,
-                Order::Ascending,
-            )
-            .collect();
-        let all = all.unwrap();
-        assert_eq!(2, all.len());
-        assert_eq!(
-            all,
-            vec![(b"jim".to_vec(), data2), (b"john".to_vec(), data.clone())]
-        );
-
-        // let's try to iterate over a more restrictive range
-        let all: StdResult<Vec<_>> = PEOPLE
-            .range_de(
-                &store,
-                Some(Bound::Inclusive(b"jo".to_vec())),
-                None,
-                Order::Ascending,
-            )
-            .collect();
-        let all = all.unwrap();
-        assert_eq!(1, all.len());
-        assert_eq!(all, vec![(b"john".to_vec(), data)]);
-    }
-
-    #[test]
-    #[cfg(feature = "iterator")]
-    fn range_de_simple_integer_key() {
-        let mut store = MockStorage::new();
-
-        // save and load on two keys
-        let data = Data {
-            name: "John".to_string(),
-            age: 32,
-        };
-        PEOPLE_ID
-            .save(&mut store, U32Key::new(1234), &data)
-            .unwrap();
-
-        let data2 = Data {
-            name: "Jim".to_string(),
-            age: 44,
-        };
-        PEOPLE_ID.save(&mut store, U32Key::new(56), &data2).unwrap();
+        PEOPLE_ID.save(&mut store, 56, &data2).unwrap();
 
         // let's try to iterate!
         let all: StdResult<Vec<_>> = PEOPLE_ID
-            .range_de(&store, None, None, Order::Ascending)
+            .range(&store, None, None, Order::Ascending)
             .collect();
         let all = all.unwrap();
         assert_eq!(2, all.len());
@@ -557,9 +547,9 @@ mod test {
 
         // let's try to iterate over a range
         let all: StdResult<Vec<_>> = PEOPLE_ID
-            .range_de(
+            .range(
                 &store,
-                Some(Bound::Inclusive(U32Key::new(56).into())),
+                Some(Bound::inclusive_int(56u32)),
                 None,
                 Order::Ascending,
             )
@@ -570,9 +560,9 @@ mod test {
 
         // let's try to iterate over a more restrictive range
         let all: StdResult<Vec<_>> = PEOPLE_ID
-            .range_de(
+            .range(
                 &store,
-                Some(Bound::Inclusive(U32Key::new(57).into())),
+                Some(Bound::inclusive_int(57u32)),
                 None,
                 Order::Ascending,
             )
@@ -582,6 +572,113 @@ mod test {
         assert_eq!(all, vec![(1234, data)]);
     }
 
+    #[test]
+    #[cfg(feature = "iterator")]
+    fn range_simple_signed_integer_key() {
+        let mut store = MockStorage::new();
+
+        // save and load on three keys
+        let data = Data {
+            name: "John".to_string(),
+            age: 32,
+        };
+        SIGNED_ID.save(&mut store, -1234, &data).unwrap();
+
+        let data2 = Data {
+            name: "Jim".to_string(),
+            age: 44,
+        };
+        SIGNED_ID.save(&mut store, -56, &data2).unwrap();
+
+        let data3 = Data {
+            name: "Jules".to_string(),
+            age: 55,
+        };
+        SIGNED_ID.save(&mut store, 50, &data3).unwrap();
+
+        // let's try to iterate!
+        let all: StdResult<Vec<_>> = SIGNED_ID
+            .range(&store, None, None, Order::Ascending)
+            .collect();
+        let all = all.unwrap();
+        assert_eq!(3, all.len());
+        // order is correct
+        assert_eq!(
+            all,
+            vec![(-1234, data), (-56, data2.clone()), (50, data3.clone())]
+        );
+
+        // let's try to iterate over a range
+        let all: StdResult<Vec<_>> = SIGNED_ID
+            .range(
+                &store,
+                Some(Bound::inclusive_int(-56i32)),
+                None,
+                Order::Ascending,
+            )
+            .collect();
+        let all = all.unwrap();
+        assert_eq!(2, all.len());
+        assert_eq!(all, vec![(-56, data2), (50, data3.clone())]);
+
+        // let's try to iterate over a more restrictive range
+        let all: StdResult<Vec<_>> = SIGNED_ID
+            .range(
+                &store,
+                Some(Bound::inclusive_int(-55i32)),
+                Some(Bound::inclusive_int(50i32)),
+                Order::Descending,
+            )
+            .collect();
+        let all = all.unwrap();
+        assert_eq!(1, all.len());
+        assert_eq!(all, vec![(50, data3)]);
+    }
+
+    #[test]
+    #[cfg(feature = "iterator")]
+    fn range_raw_composite_key() {
+        let mut store = MockStorage::new();
+
+        // save and load on three keys, one under different owner
+        ALLOWANCE
+            .save(&mut store, (b"owner", b"spender"), &1000)
+            .unwrap();
+        ALLOWANCE
+            .save(&mut store, (b"owner", b"spender2"), &3000)
+            .unwrap();
+        ALLOWANCE
+            .save(&mut store, (b"owner2", b"spender"), &5000)
+            .unwrap();
+
+        // let's try to iterate!
+        let all: StdResult<Vec<_>> = ALLOWANCE
+            .range_raw(&store, None, None, Order::Ascending)
+            .collect();
+        let all = all.unwrap();
+        assert_eq!(3, all.len());
+        assert_eq!(
+            all,
+            vec![
+                ((b"owner".to_vec(), b"spender".to_vec()).joined_key(), 1000),
+                ((b"owner".to_vec(), b"spender2".to_vec()).joined_key(), 3000),
+                ((b"owner2".to_vec(), b"spender".to_vec()).joined_key(), 5000),
+            ]
+        );
+
+        // let's try to iterate over a prefix
+        let all: StdResult<Vec<_>> = ALLOWANCE
+            .prefix(b"owner")
+            .range_raw(&store, None, None, Order::Ascending)
+            .collect();
+        let all = all.unwrap();
+        assert_eq!(2, all.len());
+        assert_eq!(
+            all,
+            vec![(b"spender".to_vec(), 1000), (b"spender2".to_vec(), 3000)]
+        );
+    }
+
     #[test]
     #[cfg(feature = "iterator")]
     fn range_composite_key() {
@@ -607,9 +704,9 @@ mod test {
         assert_eq!(
             all,
             vec![
-                ((b"owner".to_vec(), b"spender".to_vec()).joined_key(), 1000),
-                ((b"owner".to_vec(), b"spender2".to_vec()).joined_key(), 3000),
-                ((b"owner2".to_vec(), b"spender".to_vec()).joined_key(), 5000),
+                ((b"owner".to_vec(), b"spender".to_vec()), 1000),
+                ((b"owner".to_vec(), b"spender2".to_vec()), 3000),
+                ((b"owner2".to_vec(), b"spender".to_vec()), 5000)
             ]
         );
 
@@ -620,50 +717,6 @@ mod test {
             .collect();
         let all = all.unwrap();
         assert_eq!(2, all.len());
-        assert_eq!(
-            all,
-            vec![(b"spender".to_vec(), 1000), (b"spender2".to_vec(), 3000)]
-        );
-    }
-
-    #[test]
-    #[cfg(feature = "iterator")]
-    fn range_de_composite_key() {
-        let mut store = MockStorage::new();
-
-        // save and load on three keys, one under different owner
-        ALLOWANCE
-            .save(&mut store, (b"owner", b"spender"), &1000)
-            .unwrap();
-        ALLOWANCE
-            .save(&mut store, (b"owner", b"spender2"), &3000)
-            .unwrap();
-        ALLOWANCE
-            .save(&mut store, (b"owner2", b"spender"), &5000)
-            .unwrap();
-
-        // let's try to iterate!
-        let all: StdResult<Vec<_>> = ALLOWANCE
-            .range_de(&store, None, None, Order::Ascending)
-            .collect();
-        let all = all.unwrap();
-        assert_eq!(3, all.len());
-        assert_eq!(
-            all,
-            vec![
-                ((b"owner".to_vec(), b"spender".to_vec()), 1000),
-                ((b"owner".to_vec(), b"spender2".to_vec()), 3000),
-                ((b"owner2".to_vec(), b"spender".to_vec()), 5000)
-            ]
-        );
-
-        // let's try to iterate over a prefix_de
-        let all: StdResult<Vec<_>> = ALLOWANCE
-            .prefix_de(b"owner")
-            .range_de(&store, None, None, Order::Ascending)
-            .collect();
-        let all = all.unwrap();
-        assert_eq!(2, all.len());
         assert_eq!(
             all,
             vec![(b"spender".to_vec(), 1000), (b"spender2".to_vec(), 3000),]
@@ -672,44 +725,46 @@ mod test {
 
     #[test]
     #[cfg(feature = "iterator")]
-    fn range_triple_key() {
+    fn range_raw_triple_key() {
         let mut store = MockStorage::new();
 
         // save and load on three keys, one under different owner
         TRIPLE
-            .save(&mut store, (b"owner", 9u8.into(), "recipient"), &1000)
+            .save(&mut store, (b"owner", 9, "recipient"), &1000)
             .unwrap();
         TRIPLE
-            .save(&mut store, (b"owner", 9u8.into(), "recipient2"), &3000)
+            .save(&mut store, (b"owner", 9, "recipient2"), &3000)
             .unwrap();
         TRIPLE
-            .save(&mut store, (b"owner", 10u8.into(), "recipient3"), &3000)
+            .save(&mut store, (b"owner", 10, "recipient3"), &3000)
             .unwrap();
         TRIPLE
-            .save(&mut store, (b"owner2", 9u8.into(), "recipient"), &5000)
+            .save(&mut store, (b"owner2", 9, "recipient"), &5000)
             .unwrap();
 
         // let's try to iterate!
-        let all: StdResult<Vec<_>> = TRIPLE.range(&store, None, None, Order::Ascending).collect();
+        let all: StdResult<Vec<_>> = TRIPLE
+            .range_raw(&store, None, None, Order::Ascending)
+            .collect();
         let all = all.unwrap();
         assert_eq!(4, all.len());
         assert_eq!(
             all,
             vec![
                 (
-                    (b"owner".to_vec(), U8Key::new(9), b"recipient".to_vec()).joined_key(),
+                    (b"owner".to_vec(), 9u8, b"recipient".to_vec()).joined_key(),
                     1000
                 ),
                 (
-                    (b"owner".to_vec(), U8Key::new(9), b"recipient2".to_vec()).joined_key(),
+                    (b"owner".to_vec(), 9u8, b"recipient2".to_vec()).joined_key(),
                     3000
                 ),
                 (
-                    (b"owner".to_vec(), U8Key::new(10), b"recipient3".to_vec()).joined_key(),
+                    (b"owner".to_vec(), 10u8, b"recipient3".to_vec()).joined_key(),
                     3000
                 ),
                 (
-                    (b"owner2".to_vec(), U8Key::new(9), b"recipient".to_vec()).joined_key(),
+                    (b"owner2".to_vec(), 9u8, b"recipient".to_vec()).joined_key(),
                     5000
                 )
             ]
@@ -717,8 +772,8 @@ mod test {
 
         // let's iterate over a prefix
         let all: StdResult<Vec<_>> = TRIPLE
-            .prefix((b"owner", 9u8.into()))
-            .range(&store, None, None, Order::Ascending)
+            .prefix((b"owner", 9))
+            .range_raw(&store, None, None, Order::Ascending)
             .collect();
         let all = all.unwrap();
         assert_eq!(2, all.len());
@@ -733,44 +788,42 @@ mod test {
         // let's iterate over a sub prefix
         let all: StdResult<Vec<_>> = TRIPLE
             .sub_prefix(b"owner")
-            .range(&store, None, None, Order::Ascending)
+            .range_raw(&store, None, None, Order::Ascending)
             .collect();
         let all = all.unwrap();
         assert_eq!(3, all.len());
-        // Use range_de() if you want key deserialization
+        // Use range() if you want key deserialization
         assert_eq!(
             all,
             vec![
-                ((U8Key::new(9), b"recipient".to_vec()).joined_key(), 1000),
-                ((U8Key::new(9), b"recipient2".to_vec()).joined_key(), 3000),
-                ((U8Key::new(10), b"recipient3".to_vec()).joined_key(), 3000)
+                ((9u8, b"recipient".to_vec()).joined_key(), 1000),
+                ((9u8, b"recipient2".to_vec()).joined_key(), 3000),
+                ((10u8, b"recipient3".to_vec()).joined_key(), 3000)
             ]
         );
     }
 
     #[test]
     #[cfg(feature = "iterator")]
-    fn range_de_triple_key() {
+    fn range_triple_key() {
         let mut store = MockStorage::new();
 
         // save and load on three keys, one under different owner
         TRIPLE
-            .save(&mut store, (b"owner", 9u8.into(), "recipient"), &1000)
+            .save(&mut store, (b"owner", 9u8, "recipient"), &1000)
             .unwrap();
         TRIPLE
-            .save(&mut store, (b"owner", 9u8.into(), "recipient2"), &3000)
+            .save(&mut store, (b"owner", 9u8, "recipient2"), &3000)
             .unwrap();
         TRIPLE
-            .save(&mut store, (b"owner", 10u8.into(), "recipient3"), &3000)
+            .save(&mut store, (b"owner", 10u8, "recipient3"), &3000)
             .unwrap();
         TRIPLE
-            .save(&mut store, (b"owner2", 9u8.into(), "recipient"), &5000)
+            .save(&mut store, (b"owner2", 9u8, "recipient"), &5000)
             .unwrap();
 
         // let's try to iterate!
-        let all: StdResult<Vec<_>> = TRIPLE
-            .range_de(&store, None, None, Order::Ascending)
-            .collect();
+        let all: StdResult<Vec<_>> = TRIPLE.range(&store, None, None, Order::Ascending).collect();
         let all = all.unwrap();
         assert_eq!(4, all.len());
         assert_eq!(
@@ -783,10 +836,10 @@ mod test {
             ]
         );
 
-        // let's iterate over a sub_prefix_de
+        // let's iterate over a sub_prefix
         let all: StdResult<Vec<_>> = TRIPLE
-            .sub_prefix_de(b"owner")
-            .range_de(&store, None, None, Order::Ascending)
+            .sub_prefix(b"owner")
+            .range(&store, None, None, Order::Ascending)
             .collect();
         let all = all.unwrap();
         assert_eq!(3, all.len());
@@ -799,10 +852,10 @@ mod test {
             ]
         );
 
-        // let's iterate over a prefix_de
+        // let's iterate over a prefix
         let all: StdResult<Vec<_>> = TRIPLE
-            .prefix_de((b"owner", U8Key::new(9)))
-            .range_de(&store, None, None, Order::Ascending)
+            .prefix((b"owner", 9))
+            .range(&store, None, None, Order::Ascending)
             .collect();
         let all = all.unwrap();
         assert_eq!(2, all.len());
@@ -945,7 +998,7 @@ mod test {
 
     #[test]
     #[cfg(feature = "iterator")]
-    fn readme_with_range() -> StdResult<()> {
+    fn readme_with_range_raw() -> StdResult<()> {
         let mut store = MockStorage::new();
 
         // save and load on two keys
@@ -961,7 +1014,9 @@ mod test {
         PEOPLE.save(&mut store, b"jim", &data2)?;
 
         // iterate over them all
-        let all: StdResult<Vec<_>> = PEOPLE.range(&store, None, None, Order::Ascending).collect();
+        let all: StdResult<Vec<_>> = PEOPLE
+            .range_raw(&store, None, None, Order::Ascending)
+            .collect();
         assert_eq!(
             all?,
             vec![(b"jim".to_vec(), data2), (b"john".to_vec(), data.clone())]
@@ -969,7 +1024,7 @@ mod test {
 
         // or just show what is after jim
         let all: StdResult<Vec<_>> = PEOPLE
-            .range(
+            .range_raw(
                 &store,
                 Some(Bound::Exclusive(b"jim".to_vec())),
                 None,
@@ -986,7 +1041,7 @@ mod test {
         // get all under one key
         let all: StdResult<Vec<_>> = ALLOWANCE
             .prefix(b"owner")
-            .range(&store, None, None, Order::Ascending)
+            .range_raw(&store, None, None, Order::Ascending)
             .collect();
         assert_eq!(
             all?,
@@ -996,7 +1051,7 @@ mod test {
         // Or ranges between two items (even reverse)
         let all: StdResult<Vec<_>> = ALLOWANCE
             .prefix(b"owner")
-            .range(
+            .range_raw(
                 &store,
                 Some(Bound::Exclusive(b"spender1".to_vec())),
                 Some(Bound::Inclusive(b"spender2".to_vec())),
@@ -1010,46 +1065,39 @@ mod test {
 
     #[test]
     #[cfg(feature = "iterator")]
-    fn prefixed_range_works() {
+    fn prefixed_range_raw_works() {
         // this is designed to look as much like a secondary index as possible
         // we want to query over a range of u32 for the first key and all subkeys
-        const AGES: Map<(U32Key, Vec<u8>), u64> = Map::new("ages");
+        const AGES: Map<(u32, Vec<u8>), u64> = Map::new("ages");
 
         let mut store = MockStorage::new();
-        AGES.save(&mut store, (2.into(), vec![1, 2, 3]), &123)
-            .unwrap();
-        AGES.save(&mut store, (3.into(), vec![4, 5, 6]), &456)
-            .unwrap();
-        AGES.save(&mut store, (5.into(), vec![7, 8, 9]), &789)
-            .unwrap();
-        AGES.save(&mut store, (5.into(), vec![9, 8, 7]), &987)
-            .unwrap();
-        AGES.save(&mut store, (7.into(), vec![20, 21, 22]), &2002)
-            .unwrap();
-        AGES.save(&mut store, (8.into(), vec![23, 24, 25]), &2332)
-            .unwrap();
+        AGES.save(&mut store, (2, vec![1, 2, 3]), &123).unwrap();
+        AGES.save(&mut store, (3, vec![4, 5, 6]), &456).unwrap();
+        AGES.save(&mut store, (5, vec![7, 8, 9]), &789).unwrap();
+        AGES.save(&mut store, (5, vec![9, 8, 7]), &987).unwrap();
+        AGES.save(&mut store, (7, vec![20, 21, 22]), &2002).unwrap();
+        AGES.save(&mut store, (8, vec![23, 24, 25]), &2332).unwrap();
 
         // typical range under one prefix as a control
         let fives = AGES
-            .prefix(5.into())
-            .range(&store, None, None, Order::Ascending)
+            .prefix(5)
+            .range_raw(&store, None, None, Order::Ascending)
             .collect::<StdResult<Vec<_>>>()
             .unwrap();
         assert_eq!(fives.len(), 2);
         assert_eq!(fives, vec![(vec![7, 8, 9], 789), (vec![9, 8, 7], 987)]);
 
         let keys: Vec<_> = AGES
-            .no_prefix()
-            .keys(&store, None, None, Order::Ascending)
+            .keys_raw(&store, None, None, Order::Ascending)
             .collect();
         println!("keys: {:?}", keys);
 
         // using inclusive bounds both sides
         let include = AGES
-            .prefix_range(
+            .prefix_range_raw(
                 &store,
-                Some(PrefixBound::inclusive(3)),
-                Some(PrefixBound::inclusive(7)),
+                Some(PrefixBound::inclusive(3u32)),
+                Some(PrefixBound::inclusive(7u32)),
                 Order::Ascending,
             )
             .map(|r| r.map(|(_, v)| v))
@@ -1060,10 +1108,10 @@ mod test {
 
         // using exclusive bounds both sides
         let exclude = AGES
-            .prefix_range(
+            .prefix_range_raw(
                 &store,
-                Some(PrefixBound::exclusive(3)),
-                Some(PrefixBound::exclusive(7)),
+                Some(PrefixBound::exclusive(3u32)),
+                Some(PrefixBound::exclusive(7u32)),
                 Order::Ascending,
             )
             .map(|r| r.map(|(_, v)| v))
@@ -1074,10 +1122,10 @@ mod test {
 
         // using inclusive in descending
         let include = AGES
-            .prefix_range(
+            .prefix_range_raw(
                 &store,
-                Some(PrefixBound::inclusive(3)),
-                Some(PrefixBound::inclusive(5)),
+                Some(PrefixBound::inclusive(3u32)),
+                Some(PrefixBound::inclusive(5u32)),
                 Order::Descending,
             )
             .map(|r| r.map(|(_, v)| v))
@@ -1088,10 +1136,10 @@ mod test {
 
         // using exclusive in descending
         let include = AGES
-            .prefix_range(
+            .prefix_range_raw(
                 &store,
-                Some(PrefixBound::exclusive(2)),
-                Some(PrefixBound::exclusive(5)),
+                Some(PrefixBound::exclusive(2u32)),
+                Some(PrefixBound::exclusive(5u32)),
                 Order::Descending,
             )
             .map(|r| r.map(|(_, v)| v))
@@ -1103,22 +1151,22 @@ mod test {
 
     #[test]
     #[cfg(feature = "iterator")]
-    fn prefixed_range_de_works() {
+    fn prefixed_range_works() {
         // this is designed to look as much like a secondary index as possible
         // we want to query over a range of u32 for the first key and all subkeys
-        const AGES: Map<(U32Key, &str), u64> = Map::new("ages");
+        const AGES: Map<(u32, &str), u64> = Map::new("ages");
 
         let mut store = MockStorage::new();
-        AGES.save(&mut store, (2.into(), "123"), &123).unwrap();
-        AGES.save(&mut store, (3.into(), "456"), &456).unwrap();
-        AGES.save(&mut store, (5.into(), "789"), &789).unwrap();
-        AGES.save(&mut store, (5.into(), "987"), &987).unwrap();
-        AGES.save(&mut store, (7.into(), "202122"), &2002).unwrap();
-        AGES.save(&mut store, (8.into(), "232425"), &2332).unwrap();
+        AGES.save(&mut store, (2, "123"), &123).unwrap();
+        AGES.save(&mut store, (3, "456"), &456).unwrap();
+        AGES.save(&mut store, (5, "789"), &789).unwrap();
+        AGES.save(&mut store, (5, "987"), &987).unwrap();
+        AGES.save(&mut store, (7, "202122"), &2002).unwrap();
+        AGES.save(&mut store, (8, "232425"), &2332).unwrap();
 
         // typical range under one prefix as a control
         let fives = AGES
-            .prefix_de(5.into())
+            .prefix(5)
             .range(&store, None, None, Order::Ascending)
             .collect::<StdResult<Vec<_>>>()
             .unwrap();
@@ -1128,18 +1176,15 @@ mod test {
             vec![("789".to_string(), 789), ("987".to_string(), 987)]
         );
 
-        let keys: Vec<_> = AGES
-            .no_prefix()
-            .keys_de(&store, None, None, Order::Ascending)
-            .collect();
+        let keys: Vec<_> = AGES.keys(&store, None, None, Order::Ascending).collect();
         println!("keys: {:?}", keys);
 
         // using inclusive bounds both sides
         let include = AGES
-            .prefix_range_de(
+            .prefix_range(
                 &store,
-                Some(PrefixBound::inclusive(3)),
-                Some(PrefixBound::inclusive(7)),
+                Some(PrefixBound::inclusive(3u32)),
+                Some(PrefixBound::inclusive(7u32)),
                 Order::Ascending,
             )
             .map(|r| r.map(|(_, v)| v))
@@ -1150,10 +1195,10 @@ mod test {
 
         // using exclusive bounds both sides
         let exclude = AGES
-            .prefix_range_de(
+            .prefix_range(
                 &store,
-                Some(PrefixBound::exclusive(3)),
-                Some(PrefixBound::exclusive(7)),
+                Some(PrefixBound::exclusive(3u32)),
+                Some(PrefixBound::exclusive(7u32)),
                 Order::Ascending,
             )
             .map(|r| r.map(|(_, v)| v))
@@ -1164,10 +1209,10 @@ mod test {
 
         // using inclusive in descending
         let include = AGES
-            .prefix_range_de(
+            .prefix_range(
                 &store,
-                Some(PrefixBound::inclusive(3)),
-                Some(PrefixBound::inclusive(5)),
+                Some(PrefixBound::inclusive(3u32)),
+                Some(PrefixBound::inclusive(5u32)),
                 Order::Descending,
             )
             .map(|r| r.map(|(_, v)| v))
@@ -1178,10 +1223,10 @@ mod test {
 
         // using exclusive in descending
         let include = AGES
-            .prefix_range_de(
+            .prefix_range(
                 &store,
-                Some(PrefixBound::exclusive(2)),
-                Some(PrefixBound::exclusive(5)),
+                Some(PrefixBound::exclusive(2u32)),
+                Some(PrefixBound::exclusive(5u32)),
                 Order::Descending,
             )
             .map(|r| r.map(|(_, v)| v))
