@@ -4,10 +4,11 @@ use serde::Serialize;
 use cosmwasm_std::{StdError, StdResult, Storage};
 
 use crate::de::KeyDeserialize;
+use crate::iter_helpers::deserialize_kv;
 use crate::keys::PrimaryKey;
 use crate::map::Map;
 use crate::path::Path;
-use crate::prefix::Prefix;
+use crate::prefix::{namespaced_prefix_range, Prefix, PrefixBound};
 use crate::snapshot::Snapshot;
 use crate::{Bound, Prefixer, Strategy};
 
@@ -68,16 +69,8 @@ where
         self.primary.key(k)
     }
 
-    pub fn prefix(&self, p: K::Prefix) -> Prefix<Vec<u8>, T> {
-        self.primary.prefix(p)
-    }
-
-    pub fn sub_prefix(&self, p: K::SubPrefix) -> Prefix<Vec<u8>, T> {
-        self.primary.sub_prefix(p)
-    }
-
-    fn no_prefix(&self) -> Prefix<Vec<u8>, T> {
-        self.primary.no_prefix()
+    fn no_prefix_raw(&self) -> Prefix<Vec<u8>, T> {
+        self.primary.no_prefix_raw()
     }
 
     /// load old value and store changelog
@@ -163,7 +156,7 @@ where
     }
 }
 
-// short-cut for simple keys, rather than .prefix(()).range(...)
+// short-cut for simple keys, rather than .prefix(()).range_raw(...)
 impl<'a, K, T> SnapshotMap<'a, K, T>
 where
     T: Serialize + DeserializeOwned + Clone,
@@ -171,7 +164,7 @@ where
 {
     // I would prefer not to copy code from Prefix, but no other way
     // with lifetimes (create Prefix inside function and return ref = no no)
-    pub fn range<'c>(
+    pub fn range_raw<'c>(
         &self,
         store: &'c dyn Storage,
         min: Option<Bound>,
@@ -181,7 +174,20 @@ where
     where
         T: 'c,
     {
-        self.no_prefix().range(store, min, max, order)
+        self.no_prefix_raw().range_raw(store, min, max, order)
+    }
+
+    pub fn keys_raw<'c>(
+        &self,
+        store: &'c dyn Storage,
+        min: Option<Bound>,
+        max: Option<Bound>,
+        order: cosmwasm_std::Order,
+    ) -> Box<dyn Iterator<Item = Vec<u8>> + 'c>
+    where
+        T: 'c,
+    {
+        self.no_prefix_raw().keys_raw(store, min, max, order)
     }
 }
 
@@ -191,7 +197,31 @@ where
     T: Serialize + DeserializeOwned,
     K: PrimaryKey<'a> + KeyDeserialize,
 {
-    pub fn range_de<'c>(
+    /// While `range` over a `prefix` fixes the prefix to one element and iterates over the
+    /// remaining, `prefix_range` accepts bounds for the lowest and highest elements of the
+    /// `Prefix` itself, and iterates over those (inclusively or exclusively, depending on
+    /// `PrefixBound`).
+    /// There are some issues that distinguish these two, and blindly casting to `Vec<u8>` doesn't
+    /// solve them.
+    pub fn prefix_range<'c>(
+        &self,
+        store: &'c dyn Storage,
+        min: Option<PrefixBound<'a, K::Prefix>>,
+        max: Option<PrefixBound<'a, K::Prefix>>,
+        order: cosmwasm_std::Order,
+    ) -> Box<dyn Iterator<Item = StdResult<(K::Output, T)>> + 'c>
+    where
+        T: 'c,
+        'a: 'c,
+        K: 'c,
+        K::Output: 'static,
+    {
+        let mapped = namespaced_prefix_range(store, self.primary.namespace(), min, max, order)
+            .map(deserialize_kv::<K, T>);
+        Box::new(mapped)
+    }
+
+    pub fn range<'c>(
         &self,
         store: &'c dyn Storage,
         min: Option<Bound>,
@@ -202,10 +232,32 @@ where
         T: 'c,
         K::Output: 'static,
     {
-        self.no_prefix_de().range_de(store, min, max, order)
+        self.no_prefix().range(store, min, max, order)
     }
 
-    fn no_prefix_de(&self) -> Prefix<K, T> {
+    pub fn keys<'c>(
+        &self,
+        store: &'c dyn Storage,
+        min: Option<Bound>,
+        max: Option<Bound>,
+        order: cosmwasm_std::Order,
+    ) -> Box<dyn Iterator<Item = StdResult<K::Output>> + 'c>
+    where
+        T: 'c,
+        K::Output: 'static,
+    {
+        self.no_prefix().keys(store, min, max, order)
+    }
+
+    pub fn prefix(&self, p: K::Prefix) -> Prefix<K::Suffix, T> {
+        Prefix::new(self.primary.namespace(), &p.prefix())
+    }
+
+    pub fn sub_prefix(&self, p: K::SubPrefix) -> Prefix<K::SuperSuffix, T> {
+        Prefix::new(self.primary.namespace(), &p.prefix())
+    }
+
+    fn no_prefix(&self) -> Prefix<K, T> {
         Prefix::new(self.primary.namespace(), &[])
     }
 }
@@ -215,8 +267,8 @@ mod tests {
     use super::*;
     use cosmwasm_std::testing::MockStorage;
 
-    type TestMap = SnapshotMap<'static, &'static [u8], u64>;
-    type TestMapCompositeKey = SnapshotMap<'static, (&'static [u8], &'static [u8]), u64>;
+    type TestMap = SnapshotMap<'static, &'static str, u64>;
+    type TestMapCompositeKey = SnapshotMap<'static, (&'static str, &'static str), u64>;
 
     const NEVER: TestMap =
         SnapshotMap::new("never", "never__check", "never__change", Strategy::Never);
@@ -249,66 +301,58 @@ mod tests {
     // Values at beginning of 3 -> A = 5, B = 7
     // Values at beginning of 5 -> A = 8, C = 13
     fn init_data(map: &TestMap, storage: &mut dyn Storage) {
-        map.save(storage, b"A", &5, 1).unwrap();
-        map.save(storage, b"B", &7, 2).unwrap();
+        map.save(storage, "A", &5, 1).unwrap();
+        map.save(storage, "B", &7, 2).unwrap();
 
         // checkpoint 3
         map.add_checkpoint(storage, 3).unwrap();
 
         // also use update to set - to ensure this works
-        map.save(storage, b"C", &1, 3).unwrap();
-        map.update(storage, b"A", 3, |_| -> StdResult<u64> { Ok(8) })
+        map.save(storage, "C", &1, 3).unwrap();
+        map.update(storage, "A", 3, |_| -> StdResult<u64> { Ok(8) })
             .unwrap();
 
-        map.remove(storage, b"B", 4).unwrap();
-        map.save(storage, b"C", &13, 4).unwrap();
+        map.remove(storage, "B", 4).unwrap();
+        map.save(storage, "C", &13, 4).unwrap();
 
         // checkpoint 5
         map.add_checkpoint(storage, 5).unwrap();
-        map.remove(storage, b"A", 5).unwrap();
-        map.update(storage, b"D", 5, |_| -> StdResult<u64> { Ok(22) })
+        map.remove(storage, "A", 5).unwrap();
+        map.update(storage, "D", 5, |_| -> StdResult<u64> { Ok(22) })
             .unwrap();
         // and delete it later (unknown if all data present)
         map.remove_checkpoint(storage, 5).unwrap();
     }
 
-    const FINAL_VALUES: &[(&[u8], Option<u64>)] = &[
-        (b"A", None),
-        (b"B", None),
-        (b"C", Some(13)),
-        (b"D", Some(22)),
-    ];
+    const FINAL_VALUES: &[(&str, Option<u64>)] =
+        &[("A", None), ("B", None), ("C", Some(13)), ("D", Some(22))];
 
-    const VALUES_START_3: &[(&[u8], Option<u64>)] =
-        &[(b"A", Some(5)), (b"B", Some(7)), (b"C", None), (b"D", None)];
+    const VALUES_START_3: &[(&str, Option<u64>)] =
+        &[("A", Some(5)), ("B", Some(7)), ("C", None), ("D", None)];
 
-    const VALUES_START_5: &[(&[u8], Option<u64>)] = &[
-        (b"A", Some(8)),
-        (b"B", None),
-        (b"C", Some(13)),
-        (b"D", None),
-    ];
+    const VALUES_START_5: &[(&str, Option<u64>)] =
+        &[("A", Some(8)), ("B", None), ("C", Some(13)), ("D", None)];
 
-    // Same as `init_data`, but we have a composite key for testing range_de.
+    // Same as `init_data`, but we have a composite key for testing range.
     fn init_data_composite_key(map: &TestMapCompositeKey, storage: &mut dyn Storage) {
-        map.save(storage, (b"A", b"B"), &5, 1).unwrap();
-        map.save(storage, (b"B", b"A"), &7, 2).unwrap();
+        map.save(storage, ("A", "B"), &5, 1).unwrap();
+        map.save(storage, ("B", "A"), &7, 2).unwrap();
 
         // checkpoint 3
         map.add_checkpoint(storage, 3).unwrap();
 
         // also use update to set - to ensure this works
-        map.save(storage, (b"B", b"B"), &1, 3).unwrap();
-        map.update(storage, (b"A", b"B"), 3, |_| -> StdResult<u64> { Ok(8) })
+        map.save(storage, ("B", "B"), &1, 3).unwrap();
+        map.update(storage, ("A", "B"), 3, |_| -> StdResult<u64> { Ok(8) })
             .unwrap();
 
-        map.remove(storage, (b"B", b"A"), 4).unwrap();
-        map.save(storage, (b"B", b"B"), &13, 4).unwrap();
+        map.remove(storage, ("B", "A"), 4).unwrap();
+        map.save(storage, ("B", "B"), &13, 4).unwrap();
 
         // checkpoint 5
         map.add_checkpoint(storage, 5).unwrap();
-        map.remove(storage, (b"A", b"B"), 5).unwrap();
-        map.update(storage, (b"C", b"A"), 5, |_| -> StdResult<u64> { Ok(22) })
+        map.remove(storage, ("A", "B"), 5).unwrap();
+        map.update(storage, ("C", "A"), 5, |_| -> StdResult<u64> { Ok(22) })
             .unwrap();
         // and delete it later (unknown if all data present)
         map.remove_checkpoint(storage, 5).unwrap();
@@ -324,7 +368,7 @@ mod tests {
         map: &TestMap,
         storage: &dyn Storage,
         height: u64,
-        values: &[(&[u8], Option<u64>)],
+        values: &[(&str, Option<u64>)],
     ) {
         for (k, v) in values.iter().cloned() {
             assert_eq!(v, map.may_load_at_height(storage, k, height).unwrap());
@@ -332,7 +376,7 @@ mod tests {
     }
 
     fn assert_missing_checkpoint(map: &TestMap, storage: &dyn Storage, height: u64) {
-        for k in &[b"A", b"B", b"C", b"D"] {
+        for k in &["A", "B", "C", "D"] {
             assert!(map.may_load_at_height(storage, *k, height).is_err());
         }
     }
@@ -378,79 +422,59 @@ mod tests {
         let mut storage = MockStorage::new();
 
         println!("SETUP");
-        EVERY.save(&mut storage, b"A", &5, 1).unwrap();
-        EVERY.save(&mut storage, b"B", &7, 2).unwrap();
-        EVERY.save(&mut storage, b"C", &2, 2).unwrap();
+        EVERY.save(&mut storage, "A", &5, 1).unwrap();
+        EVERY.save(&mut storage, "B", &7, 2).unwrap();
+        EVERY.save(&mut storage, "C", &2, 2).unwrap();
 
         // update and save - A query at 3 => 5, at 4 => 12
         EVERY
-            .update(&mut storage, b"A", 3, |_| -> StdResult<u64> { Ok(9) })
+            .update(&mut storage, "A", 3, |_| -> StdResult<u64> { Ok(9) })
             .unwrap();
-        EVERY.save(&mut storage, b"A", &12, 3).unwrap();
-        assert_eq!(
-            Some(5),
-            EVERY.may_load_at_height(&storage, b"A", 2).unwrap()
-        );
-        assert_eq!(
-            Some(5),
-            EVERY.may_load_at_height(&storage, b"A", 3).unwrap()
-        );
+        EVERY.save(&mut storage, "A", &12, 3).unwrap();
+        assert_eq!(Some(5), EVERY.may_load_at_height(&storage, "A", 2).unwrap());
+        assert_eq!(Some(5), EVERY.may_load_at_height(&storage, "A", 3).unwrap());
         assert_eq!(
             Some(12),
-            EVERY.may_load_at_height(&storage, b"A", 4).unwrap()
+            EVERY.may_load_at_height(&storage, "A", 4).unwrap()
         );
 
         // save and remove - B query at 4 => 7, at 5 => None
-        EVERY.save(&mut storage, b"B", &17, 4).unwrap();
-        EVERY.remove(&mut storage, b"B", 4).unwrap();
-        assert_eq!(
-            Some(7),
-            EVERY.may_load_at_height(&storage, b"B", 3).unwrap()
-        );
-        assert_eq!(
-            Some(7),
-            EVERY.may_load_at_height(&storage, b"B", 4).unwrap()
-        );
-        assert_eq!(None, EVERY.may_load_at_height(&storage, b"B", 5).unwrap());
+        EVERY.save(&mut storage, "B", &17, 4).unwrap();
+        EVERY.remove(&mut storage, "B", 4).unwrap();
+        assert_eq!(Some(7), EVERY.may_load_at_height(&storage, "B", 3).unwrap());
+        assert_eq!(Some(7), EVERY.may_load_at_height(&storage, "B", 4).unwrap());
+        assert_eq!(None, EVERY.may_load_at_height(&storage, "B", 5).unwrap());
 
         // remove and update - C query at 5 => 2, at 6 => 16
-        EVERY.remove(&mut storage, b"C", 5).unwrap();
+        EVERY.remove(&mut storage, "C", 5).unwrap();
         EVERY
-            .update(&mut storage, b"C", 5, |_| -> StdResult<u64> { Ok(16) })
+            .update(&mut storage, "C", 5, |_| -> StdResult<u64> { Ok(16) })
             .unwrap();
-        assert_eq!(
-            Some(2),
-            EVERY.may_load_at_height(&storage, b"C", 4).unwrap()
-        );
-        assert_eq!(
-            Some(2),
-            EVERY.may_load_at_height(&storage, b"C", 5).unwrap()
-        );
+        assert_eq!(Some(2), EVERY.may_load_at_height(&storage, "C", 4).unwrap());
+        assert_eq!(Some(2), EVERY.may_load_at_height(&storage, "C", 5).unwrap());
         assert_eq!(
             Some(16),
-            EVERY.may_load_at_height(&storage, b"C", 6).unwrap()
+            EVERY.may_load_at_height(&storage, "C", 6).unwrap()
         );
     }
 
     #[test]
     #[cfg(feature = "iterator")]
-    fn range_de_simple_string_key() {
+    fn range_simple_string_key() {
         use cosmwasm_std::Order;
 
         let mut store = MockStorage::new();
         init_data(&EVERY, &mut store);
 
         // let's try to iterate!
-        let all: StdResult<Vec<_>> = EVERY
-            .range_de(&store, None, None, Order::Ascending)
-            .collect();
+        let all: StdResult<Vec<_>> = EVERY.range(&store, None, None, Order::Ascending).collect();
         let all = all.unwrap();
         assert_eq!(2, all.len());
-        assert_eq!(all, vec![(b"C".to_vec(), 13), (b"D".to_vec(), 22)]);
+        assert_eq!(all, vec![("C".into(), 13), ("D".into(), 22)]);
 
         // let's try to iterate over a range
         let all: StdResult<Vec<_>> = EVERY
-            .range_de(
+            .range(
                 &store,
                 Some(Bound::Inclusive(b"C".to_vec())),
                 None,
@@ -459,11 +483,11 @@ mod tests {
             .collect();
         let all = all.unwrap();
         assert_eq!(2, all.len());
-        assert_eq!(all, vec![(b"C".to_vec(), 13), (b"D".to_vec(), 22)]);
+        assert_eq!(all, vec![("C".into(), 13), ("D".into(), 22)]);
 
         // let's try to iterate over a more restrictive range
         let all: StdResult<Vec<_>> = EVERY
-            .range_de(
+            .range(
                 &store,
                 Some(Bound::Inclusive(b"D".to_vec())),
                 None,
@@ -472,12 +496,12 @@ mod tests {
             .collect();
         let all = all.unwrap();
         assert_eq!(1, all.len());
-        assert_eq!(all, vec![(b"D".to_vec(), 22)]);
+        assert_eq!(all, vec![("D".into(), 22)]);
     }
 
     #[test]
     #[cfg(feature = "iterator")]
-    fn range_de_composite_key() {
+    fn range_composite_key() {
         use cosmwasm_std::Order;
 
         let mut store = MockStorage::new();
@@ -485,15 +509,81 @@ mod tests {
 
         // let's try to iterate!
         let all: StdResult<Vec<_>> = EVERY_COMPOSITE_KEY
-            .range_de(&store, None, None, Order::Ascending)
+            .range(&store, None, None, Order::Ascending)
             .collect();
         let all = all.unwrap();
         assert_eq!(2, all.len());
         assert_eq!(
             all,
             vec![
-                ((b"B".to_vec(), b"B".to_vec()), 13),
-                ((b"C".to_vec(), b"A".to_vec()), 22)
+                (("B".into(), "B".into()), 13),
+                (("C".into(), "A".into()), 22)
+            ]
+        );
+    }
+
+    #[test]
+    #[cfg(feature = "iterator")]
+    fn prefix_range_composite_key() {
+        use cosmwasm_std::Order;
+
+        let mut store = MockStorage::new();
+        init_data_composite_key(&EVERY_COMPOSITE_KEY, &mut store);
+
+        // let's prefix-range and iterate
+        let all: StdResult<Vec<_>> = EVERY_COMPOSITE_KEY
+            .prefix_range(
+                &store,
+                None,
+                Some(PrefixBound::exclusive("C")),
+                Order::Descending,
+            )
+            .collect();
+        let all = all.unwrap();
+        assert_eq!(1, all.len());
+        assert_eq!(all, vec![(("B".into(), "B".into()), 13)]);
+    }
+
+    #[test]
+    #[cfg(feature = "iterator")]
+    fn prefix_composite_key() {
+        use cosmwasm_std::Order;
+
+        let mut store = MockStorage::new();
+        init_data_composite_key(&EVERY_COMPOSITE_KEY, &mut store);
+
+        // let's prefix and iterate
+        let all: StdResult<Vec<_>> = EVERY_COMPOSITE_KEY
+            .prefix("C")
+            .range(&store, None, None, Order::Ascending)
+            .collect();
+        let all = all.unwrap();
+        assert_eq!(1, all.len());
+        assert_eq!(all, vec![("A".into(), 22),]);
+    }
+
+    #[test]
+    #[cfg(feature = "iterator")]
+    fn sub_prefix_composite_key() {
+        use cosmwasm_std::Order;
+
+        let mut store = MockStorage::new();
+        init_data_composite_key(&EVERY_COMPOSITE_KEY, &mut store);
+
+        // Let's sub-prefix and iterate.
+        // This is similar to calling range() directly, but added here for completeness /
+        // sub_prefix type checks
+        let all: StdResult<Vec<_>> = EVERY_COMPOSITE_KEY
+            .sub_prefix(())
+            .range(&store, None, None, Order::Ascending)
+            .collect();
+        let all = all.unwrap();
+        assert_eq!(2, all.len());
+        assert_eq!(
+            all,
+            vec![
+                (("B".into(), "B".into()), 13),
+                (("C".into(), "A".into()), 22)
             ]
         );
     }
